import logging
import os
from typing import List, Dict
from pathlib import Path

from appsec_discovery.parsers import Parser
from appsec_discovery.models import CodeObject, CodeObjectProp, CodeObjectField

logger = logging.getLogger(__name__)

class JavaParser(Parser):

    def run_scan(self) -> List[CodeObject]:

        objects_list: List[CodeObject] = []

        parser_folder = str(Path(__file__).resolve().parent)

        rules_folder = os.path.join(parser_folder, "scanner_rules")

        if os.path.isdir(rules_folder):
            
            semgrep_data = self.run_semgrep(source_folder=self.source_folder, rules_folder=rules_folder)
            objects_list = self.parse_report(semgrep_data)
        
        return objects_list


    def parse_report(self, semgrep_data) -> List[CodeObject]:

        parsed_objects: List[CodeObject] = []
        parsed_objects_dict: Dict[str, CodeObject] = {}
        parsed_fields_dict = {}

        for finding in semgrep_data:

            finding_file = finding.get('path')
            finding_line_int = finding.get('start',{}).get('line',0)

            rule_id = finding.get('check_id',"").split('.')[-1]

            object_type = None

            # parse dto rules
            if rule_id.startswith("dto-"):

                object_type="dto"
                orm_type = rule_id.split('-')[1]

                object = finding.get('extra').get('metavars').get('$OBJECT', {}).get('abstract_content',"").strip('"').strip("'")
                db_table = finding.get('extra').get('metavars').get('$DB_NAME', {}).get('abstract_content',"").strip('"').strip("'")
                db_field = finding.get('extra').get('metavars').get('$DB_FIELD', {}).get('abstract_content',"").strip('"').strip("'")       
                field_type = finding.get('extra').get('metavars').get('$TYPE', {}).get('abstract_content',"").strip('"').strip("'")

                hash_key = self.calc_uniq_hash([finding_file, object])
                field_hash_key = self.calc_uniq_hash([finding_file, object, db_field])

                if hash_key not in parsed_objects_dict:

                    parsed_objects_dict[hash_key] = CodeObject(
                        hash=hash_key,
                        object_name=f"{orm_type.title()} {object_type} {object}",
                        object_type=object_type,
                        parser=self.parser,
                        file=finding_file,
                        line=finding_line_int,
                        properties={},
                        fields={}
                    )

                    parsed_objects_dict[hash_key].properties['framework'] = CodeObjectProp(
                        prop_name='framework',
                        prop_value=orm_type
                    )

   
                    parsed_objects_dict[hash_key].properties['object'] = CodeObjectProp(
                        prop_name='object',
                        prop_value=object
                    )

                    if db_table:
                        parsed_objects_dict[hash_key].properties['db_table'] = CodeObjectProp(
                            prop_name='db_table',
                            prop_value=db_table
                        )

                if field_hash_key not in parsed_fields_dict:

                    parsed_field = CodeObjectField(
                        field_name=db_field,
                        field_type=field_type,
                        file=finding_file,
                        line=finding_line_int
                    )

                    parsed_objects_dict[hash_key].fields[db_field] = parsed_field
                    parsed_fields_dict[field_hash_key] = parsed_field

        
        if parsed_objects_dict:
            parsed_objects = list(parsed_objects_dict.values())

        logger.info(f"For scan {self.parser} data parse {len(parsed_objects)} objects")
        return parsed_objects



import logging
import subprocess
import json
import hashlib
from abc import ABC, abstractmethod
from typing import List
from appsec_discovery.models import CodeObject

logger = logging.getLogger(__name__)

class Parser(ABC):

    def __init__(self, parser, source_folder):

        self.parser = parser
        self.source_folder = source_folder

    @abstractmethod
    def parse_report(self, scanner_data) -> List[CodeObject]:
        pass

    @abstractmethod
    def run_scan(self) -> List[CodeObject]:
        pass

    def run_semgrep(self, source_folder: str, rules_folder: str):

        logger.info(f"Start {self.parser} scan for {source_folder}")

        try:
            result = subprocess.run(
                ["semgrep", "scan", "-f", rules_folder, "--json", "--metrics=off", "--no-git-ignore", source_folder],
                capture_output=True,
                text=True
            )

            if 'results' in result.stdout:

                semgrep_data = json.loads(result.stdout)

                normalized_results = []

                # save relative path
                for finding in semgrep_data.get('results',[]):

                    path = finding.get('path','')
                    finding['path'] = path.replace(source_folder, '')
                    normalized_results.append(finding)

                logger.info(f"End {self.parser} scan for {source_folder}, found {len(normalized_results)} rule hits")

                return normalized_results
            
            else:
                logger.error(f"Failed {self.parser} scan for {source_folder}: {result.stderr}")
        
        except Exception as ex:
            logger.error(f"Failed {self.parser} scan for {source_folder}: {ex}")
            
        return None        


    def calc_uniq_hash(self, prop_list: List[str]) -> str:

        hash_str = "#".join(prop_list)
        hash = hashlib.md5(hash_str.encode('utf-8')).hexdigest()

        return hash



from typing import List
import logging
import yaml
import re

from appsec_discovery.models import ScoreConfig, CodeObject
from appsec_discovery.parsers import ParserFactory, Parser
from appsec_discovery.services.ai_service import AiService

logger = logging.getLogger(__name__)

severities_int = {'critical': 5, 'high': 4, 'medium': 3, 'low': 2, 'info': 1}

class ScanService:

    def __init__(self, source_folder=None, conf_file=None, only_scored_objects=False):

        self.conf_file = conf_file
        self.source_folder = source_folder

        self.config = None

        if conf_file:
            self.config = self.load_conf_from_yaml(conf_file)
        else:
            self.config = ScoreConfig()

        self.only_scored_objects = only_scored_objects

    def load_conf_from_yaml(self, score_config_file_stream):

        try:

            config_data = yaml.safe_load(score_config_file_stream)
            score_config = ScoreConfig(**config_data)

            if score_config :
                return score_config
                
        except Exception as ex:
            logger.error(f"Failed to load config scan folder {self.source_folder}: {ex}")

    def scan_folder(self) -> List[CodeObject]:

        parsed_objects: List[CodeObject] = []

        all_parsers = ParserFactory.get_parser_types()

        parsers_to_scan = []

        if 'all' in self.config.parsers:
            parsers_to_scan = all_parsers
        else:
            for parser in self.config.parsers:
                if parser in all_parsers:
                    parsers_to_scan.append(parser)

        for parser in parsers_to_scan:

            ParserCls = ParserFactory.get_parser(parser)
            parser_instance: Parser = ParserCls(parser=parser, source_folder=self.source_folder)

            res = parser_instance.run_scan()

            if res:
                parsed_objects += res

        filtered_objects = self.filter_objects(parsed_objects)
        scored_objects = self.score_objects(filtered_objects)

        if self.config.ai_params :
            ai = AiService(self.config.ai_params, self.config.exclude_scoring)
            ai_scored_objects = ai.ai_score_objects(scored_objects)

            result_objects = [ obj for obj in ai_scored_objects if obj.severity or not self.only_scored_objects ]
            return result_objects

        result_objects = [ obj for obj in scored_objects if obj.severity or not self.only_scored_objects ]
        return result_objects

    
    def filter_objects(self, parsed_objects: List[CodeObject]):

        filtered_objects: List[CodeObject] = []

        for object in parsed_objects:

            fitered = False

            for exclude in self.config.exclude_scan:

                if ( exclude.file or exclude.object_name or exclude.object_type or exclude.parser ) \
                    and (exclude.parser is None or exclude.parser.lower() == object.parser.lower()) \
                    and (exclude.file is None or re.match(exclude.file, object.file) or exclude.file.lower() in object.file.lower()) \
                    and (exclude.object_name is None or re.match(exclude.object_name, object.object_name) or exclude.object_name.lower() in object.object_name.lower()) \
                    and (exclude.object_type is None or re.match(exclude.object_type, object.object_type) or exclude.object_type.lower() in object.object_type.lower()) :
                    
                    fitered = True

            if not fitered:
                filtered_objects.append(object)

        return filtered_objects
    

    def score_objects(self, filtered_objects: List[CodeObject]):

        scored_objects: List[CodeObject] = []

        for object in filtered_objects:

            for tag, severities in self.config.score_tags.items():
                for severity, keywords in severities.items():
                    for keyword in keywords:

                        for prop_name, prop in object.properties.items():

                            if re.match(keyword, prop.prop_value) or keyword.lower() in prop.prop_value.lower():

                                excluded = False

                                for exclude in self.config.exclude_scoring:

                                    if ( exclude.file or exclude.parser or exclude.object_name or exclude.object_type or exclude.prop_name 
                                         or exclude.field_name or exclude.field_type or exclude.tag or exclude.keyword ) \
                                        and (exclude.parser is None or exclude.parser.lower() == object.parser.lower()) \
                                        and (exclude.file is None or re.match(exclude.file, object.file) or exclude.file.lower() in object.file.lower()) \
                                        and (exclude.object_name is None or re.match(exclude.object_name, object.object_name) or exclude.object_name.lower() in object.object_name.lower()) \
                                        and (exclude.object_type is None or re.match(exclude.object_type, object.object_type) or exclude.object_type.lower() in object.object_type.lower()) \
                                        and (exclude.prop_name is None or re.match(exclude.prop_name, prop_name) or exclude.prop_name.lower() in prop_name.lower()) \
                                        and (exclude.field_name is None ) \
                                        and (exclude.field_type is None ) \
                                        and (exclude.tag is None or exclude.tag == tag ) \
                                        and (exclude.keyword is None or exclude.tag == keyword) :
                                        
                                        excluded = True

                                if not excluded :

                                    if not object.properties[prop_name].severity:
                                        object.properties[prop_name].severity = severity
                                        object.properties[prop_name].tags = [tag]                                            
                                    else:
                                        if tag not in object.properties[prop_name].tags:
                                            object.properties[prop_name].tags.append(tag)

                                        if severities_int[severity] > severities_int[object.properties[prop_name].severity]:
                                            object.properties[prop_name].severity = severity

                                    if not object.severity:
                                        object.severity = severity
                                        object.tags=[tag]
                                    else:
                                        if tag not in object.tags:
                                            object.tags.append(tag)

                                        if severities_int[severity] > severities_int[object.severity]:
                                            object.severity = severity

                        scored_fields = {}

                        for field_name, field in object.fields.items():

                            if re.match(keyword, field.field_name) or keyword.lower() in field.field_name.lower():

                                excluded = False

                                for exclude in self.config.exclude_scoring:

                                    if ( exclude.file or exclude.parser or exclude.object_name or exclude.object_type or exclude.prop_name 
                                         or exclude.field_name or exclude.field_type or exclude.tag or exclude.keyword ) \
                                        and (exclude.parser is None or exclude.parser.lower() == object.parser.lower()) \
                                        and (exclude.file is None or re.match(exclude.file, object.file) or exclude.file.lower() in object.file.lower()) \
                                        and (exclude.object_name is None or re.match(exclude.object_name, object.object_name) or exclude.object_name.lower() in object.object_name.lower()) \
                                        and (exclude.object_type is None or re.match(exclude.object_type, object.object_type) or exclude.object_type.lower() in object.object_type.lower()) \
                                        and (exclude.prop_name is None ) \
                                        and (exclude.field_name is None or re.match(exclude.field_name, field.field_name) or exclude.field_name.lower() in field.field_name.lower()) \
                                        and (exclude.field_type is None or re.match(exclude.field_type, field.field_type) or exclude.field_type.lower() in field.field_type.lower()) \
                                        and (exclude.tag is None or exclude.tag == tag ) \
                                        and (exclude.keyword is None or exclude.tag == keyword) :
                                        
                                        excluded = True

                                if not excluded :

                                    if not field.severity:
                                        field.severity = severity
                                        field.tags = [tag]
                                    else:
                                        if tag not in field.tags:
                                            field.tags.append(tag)

                                        if severities_int[severity] > severities_int[field.severity]:
                                            field.severity = severity

                                    if not object.severity:
                                        object.severity = severity
                                        object.tags = [tag]
                                    else:
                                        if tag not in object.tags:
                                            object.tags.append(tag)

                                        if severities_int[severity] > severities_int[object.severity]:
                                            object.severity = severity
  
                            scored_fields[field_name] = field
                        
                        object.fields = scored_fields

            scored_objects.append(object)

        return scored_objects



from pydantic import BaseModel

class DefectdojoImportScanRequest(BaseModel):
    key: str
    name: str
    type: str
    file: int
    line: int
    properties: str
    fields: str
    score: str

class DefectdojoProjectTypeRequest(BaseModel):
    key: str
    name: str

class DiscoveryImportScanRequest(BaseModel):
    key: str
    name: str
    type: str


from pydantic import BaseModel
from typing import List, Dict


class SarifReport(BaseModel):
    file: str
    object_type: str
    object_name: str

class JsonReport(BaseModel):
    file: str
    object_type: str
    object_name: str

class DiffReport(BaseModel):
    file: str
    object_type: str
    object_name: str




import logging
import os
from typing import List
from pathlib import Path

from appsec_discovery.parsers import Parser
from appsec_discovery.models import CodeObject

logger = logging.getLogger(__name__)

class GraphqlParser(Parser):

    def run_scan(self) -> List[CodeObject]:

        objects_list: List[CodeObject] = []

        parser_folder = str(Path(__file__).resolve().parent)

        rules_folder = os.path.join(parser_folder, "scanner_rules")

        if os.path.isdir(rules_folder):
            
            semgrep_data = self.run_semgrep(source_folder=self.source_folder, rules_folder=rules_folder)
            objects_list = self.parse_report(semgrep_data)
        
        return objects_list


    def parse_report(self, semgrep_data) -> List[CodeObject]:

        parsed_objects: List[CodeObject] = []

        '''

        objects_dict = {}

        for finding in semgrep_data:

            finding_file = finding.get('path')
            finding_line_int = finding.get('start',{}).get('line',0)

            type = finding.get('extra').get('metavars').get('$TYPE', {}).get('abstract_content',"").strip("\"'![]")
            type_name = finding.get('extra').get('metavars').get('$TYPENAME', {}).get('abstract_content',"").strip("\"'![]")
            field_name = finding.get('extra').get('metavars').get('$FIELDNAME', {}).get('abstract_content',"").strip("\"'![]")
            input_name = finding.get('extra').get('metavars').get('$INPUTNAME', {}).get('abstract_content',"").strip("\"'![]")
            field_type = finding.get('extra').get('metavars').get('$FIELDTYPENAME', {}).get('abstract_content',"").strip("\"'![]")

            if type_name not in objects_dict:
                objects_dict[type_name] = {} # type fields

            if field_name not in objects_dict[type_name]:
                objects_dict[type_name][field_name] = {'input': input_name, 'type': field_type, 'file': finding_file, 'line': finding_line_int}

        if 'Query' in objects_dict and len(objects_dict['Query']) == 1 and objects_dict['Query'][0]['input'] is None:
            queries_root = 

        ''' 
        
        logger.info(f"For scan {self.parser} data parse {len(parsed_objects)} objects")
        return parsed_objects


from appsec_discovery.models.code_object import CodeObject, CodeObjectProp, CodeObjectField
from appsec_discovery.models.config import ScoreConfig, ExcludeScoring, AiParams
from appsec_discovery.models.reports import JsonReport, DiffReport, SarifReport
from appsec_discovery.models.uploads import DefectdojoImportScanRequest, DefectdojoProjectTypeRequest, DiscoveryImportScanRequest


from pydantic import BaseModel
from typing import List, Dict, Optional

class ExcludeScan(BaseModel):
    file: Optional[str]
    parser: Optional[str]
    object_name: Optional[str]
    object_type: Optional[str]

class ExcludeScoring(BaseModel):
    file: Optional[str]
    parser: Optional[str]
    object_name: Optional[str]
    object_type: Optional[str]
    prop_name: Optional[str]
    field_name: Optional[str]
    field_type: Optional[str]
    tag: Optional[str]
    keyword: Optional[str]

class AiParams(BaseModel):
    model_id: str
    model_folder: str
    gguf_file: str
    prompt: str

class ScoreConfig(BaseModel):
    parsers: List[str] = ['all']
    object_types: List[str] = ['all']
    score_tags: Dict[str,Dict[str,List[str]]] = {}
    ai_params: Optional[AiParams]
    exclude_scan: List[ExcludeScan] = []
    exclude_scoring: List[ExcludeScoring] = []




from typing import List, Dict
import logging
import yaml
import json
import sarif_om as om
from jschema_to_python.to_json import to_json

from appsec_discovery.models import CodeObject

logger = logging.getLogger(__name__)


class ReportService:

    def __init__(self, code_objects: List[CodeObject], report_type, report_file):

        self.report_type = report_type
        self.report_file = report_file

        self.code_objects = code_objects


    def save_report_to_disk(self):

        if self.report_type == 'yaml':
            report_str = self.get_yaml_report()

        if self.report_type == 'json':
            report_str = self.get_json_report()

        if self.report_type == 'sarif':
            report_str = self.get_sarif_report()

        if self.report_file :

            self.report_file.write(report_str)
            self.report_file.close()

        else:
            print(report_str)


    def get_json_report(self):

        dumped_objects = []

        for object in self.code_objects:
            dumped_objects.append(object.dict(exclude_none=True))
            
        return json.dumps(dumped_objects, indent=4)
    
    def get_yaml_report(self):

        dumped_objects = []

        for object in self.code_objects:
            dumped_objects.append(object.dict(exclude_none=True))
            
        return yaml.dump(dumped_objects, default_flow_style=False, sort_keys=False)


    def get_sarif_report(self):

        report = om.SarifLog(
            schema_uri="https://json.schemastore.org/sarif-2.1.0.json",
            version="2.1.0",
            runs=[
                om.Run(
                    tool=om.Tool(
                        driver=om.ToolComponent(
                            name="appsec-discovery",
                            semantic_version="0.1.0",
                            information_uri="https://github.com/dmarushkin/appsec-discovery",
                            rules=[]
                        )
                    ),
                    results=[]
                )
            ]
        )

        rules: Dict[str, om.ReportingDescriptor] = {}

        for object in self.code_objects:

            rule = om.ReportingDescriptor(
                id=f"{object.parser}.{object.object_type}",
                short_description={
                    "text": f"Discovered object {object.parser}.{object.object_type}"
                },
                full_description={
                    "text": f"Discovered object {object.parser}.{object.object_type}"
                },
                help={
                    "text": f"Discovered object {object.parser}.{object.object_type}"
                },
                properties={},
                default_configuration={},
            )

            rules[f"{object.parser}.{object.object_type}"] = rule

        report.runs[0].tool.driver.rules = [rule for rule in rules.values()]

        for object in self.code_objects:

            object_snippet = yaml.safe_dump(object.dict(exclude_none=True), sort_keys=False)

            region = om.Region(
                start_line=object.line,
            )
            context_region=om.Region(
                start_line=object.line,
                snippet=om.ArtifactContent(text=object_snippet)
            )

            level = 'note'
            properties = {}
            severity = 'info'

            if object.severity and object.severity in ['critical','high']:
                level = 'error'
                severity = object.severity
            
            if object.severity and object.severity in ['medium','low']:
                level = 'warning'
                severity = object.severity

            if object.severity and object.tags :
                properties = {"tags": object.tags }

            result = om.Result(
                rule_id=f"{object.parser}.{object.object_type}",
                level=level,
                properties=properties,
                message=om.Message(
                    text=f"[{object.parser}.{object.object_type}] {object.object_name}"
                ),
                locations=[
                    om.Location(
                        physical_location=om.PhysicalLocation(
                            artifact_location=om.ArtifactLocation(
                                uri=object.file,
                                uri_base_id="%SRCROOT%"
                            ),
                            region=region,
                            context_region=context_region
                        )
                    )
                ]
            )

            report.runs[0].results.append(result)

        return to_json(report)

    


import logging
import os
from typing import List
from pathlib import Path

from appsec_discovery.parsers import Parser
from appsec_discovery.models import CodeObject

logger = logging.getLogger(__name__)

class DbParser(Parser):

    def run_scan(self) -> List[CodeObject]:

        objects_list: List[CodeObject] = []

        parser_folder = str(Path(__file__).resolve().parent)

        rules_folder = os.path.join(parser_folder, "scanner_rules")

        if os.path.isdir(rules_folder):
            
            semgrep_data = self.run_semgrep(source_folder=self.source_folder, rules_folder=rules_folder)
            objects_list = self.parse_report(semgrep_data)
        
        return objects_list


    def parse_report(self, semgrep_data) -> List[CodeObject]:

        parsed_objects: List[CodeObject] = []


        logger.info(f"For scan {self.parser} data parse {len(parsed_objects)} objects")
        return parsed_objects